@@ -2,8 +2,7 @@
 #ifndef _acbuf_H
 #define _acbuf_H
 
-//#include <sys/types.h>
-//#include <limits.h>
+#include <limits.h>
 #include <string.h>
 #include "meta.h"
 
@@ -65,7 +64,7 @@ class acbuf {
         char *m_buf;
 
     	acbuf(const acbuf&); // don't copy me
-        
+    	acbuf& operator=(const acbuf&); 
         
 };
 
