@@ -15,9 +15,16 @@ namespace aclog {
       void flush();
       
       void GenerateReport(MYSTD::string &);
+      typedef struct {
+      	time_t from, to;
+      	double count, ratioSent;
+      } tRowData;
 
+      void GetStats(MYSTD::vector<tRowData> & out);
 }
 
+#define TIMEFORMAT "%a %d/%m"
+
 #endif
 
 
