@@ -20,6 +20,14 @@
 #define WEAK_PTR SMARTPTR_SPACE::weak_ptr
 #define SCOPED_PTR MYSTD::auto_ptr
 
+#ifdef NO_EXCEPTIONS
+#define MYTRY
+#define MYCATCH(x) if(false)
+#else
+#define MYTRY try
+#define MYCATCH catch
+#endif
+
 //#define atooff_t atoi
 #define tStrPos MYSTD::string::size_type 
 #define stmiss MYSTD::string::npos
@@ -48,6 +56,5 @@
 
 #endif
 
-
 #endif // __CONFIG_H
 
