@@ -47,10 +47,10 @@ public:
 	virtual bool StoreFileData(const char *data, unsigned int size);
 	virtual header const * GetHeader();
 	
-	bool m_bIsDynType;
+	bool m_bCheckFreshness;
 	volatile FiStatus status;
 	uint64_t m_nIncommingCount;
-	off_t m_nFileSize;
+	off_t m_nSizeChecked, m_nSizeSeen;
 	MYSTD::string m_sPath, m_sKey;
 	
 protected:
