@@ -14,8 +14,6 @@ public:
 	/// Override only when really needed! Instead, use Action for contents of regular pages! 
 	virtual void Run(const MYSTD::string &cmd)=0;
 
-	bool m_bErrAbort;
-	
 protected:
 	
 	virtual void SendMsg(const MYSTD::string &x);
@@ -33,13 +31,6 @@ private:
 	maintenance& operator=(const maintenance&);
 };
 
-class reportgen : public maintenance
-{
-public:
-	reportgen(int);
-	virtual void Run(const MYSTD::string &);
-	virtual void GenReport();
-};
 
 void DispatchAndRunMaintTask(const MYSTD::string &, int, const char*);
 
