@@ -3,9 +3,9 @@
 
 #include <string>
 #include <map>
+#include <set>
 #include <vector>
 
-#define _XOPEN_SOURCE 600
 #include <fcntl.h>
 #include <pthread.h>
 #include <strings.h>
@@ -120,6 +120,18 @@ static inline char * mempbrk (char * membuf, char const * const needles, size_t
 }
 
 typedef MYSTD::vector<MYSTD::string> tStrVec;
+/*
+template<class T>
+bool VectorContains(const MYSTD::vector<T> haystack, const T needle)
+{
+	for(unsigned long i=0; i<haystack.size();i++)
+	{
+		if(needle==haystack[i])
+			return true;
+	}
+	return false;
+}
+*/
 
 // Sometimes I miss Perl...
 tStrVec::size_type Tokenize(const MYSTD::string &in, const char *sep, tStrVec & out);
@@ -151,6 +163,8 @@ void appendLong(MYSTD::string &s, long val);
 
 #define OFFT_MAX (std::numeric_limits<off_t>::max())
 
+bool FileCopy(const MYSTD::string &from, const MYSTD::string to);
+
 #endif
 
 
