@@ -23,14 +23,16 @@ bool acbuf::init(unsigned int c) {
 			delete [] m_buf;
 			m_buf=NULL;
 		}
-		try
+		MYTRY
 		{
 			m_buf=new char[c+1];
 		}
-		catch(MYSTD::bad_alloc)
+		MYCATCH(MYSTD::bad_alloc)
 		{
 			return false;
 		}
+		if(!m_buf)
+			return false;
 	}
     m_buf[c]=0; // terminate to make string operations safe
     r=w=0;
@@ -66,20 +68,28 @@ bool acbuf::initFromFile(const char *szPath)
 int acbuf::syswrite(int fd, unsigned int maxlen) {
     size_t todo(MYSTD::min(maxlen, size()));
 
-    int n=::write(fd, rptr(), todo);
-    if(n<0 && errno==EAGAIN)
-        n=0;
+	int n;
+	do
+	{
+		n=::write(fd, rptr(), todo);
+	} while(n<0 && errno==EINTR);
+	
+	if(n<0 && errno==EAGAIN)
+		n=0;
     if(n<0)
-        return n;
+        return -errno;
     drop(n);
     return n;
 }
 
 int acbuf::sysread(int fd) {
-    int n=::read(fd, m_buf+w, m_nCapacity-w);
+	int n;
+	do {
+		n=::read(fd, m_buf+w, m_nCapacity-w);
+	} while(n<0 && errno==EINTR);
     //cerr << "sysread: " << free() <<endl;
-    if(n<0 && errno==EAGAIN)
-        n=0;
+    if(n<0)
+    	return -errno;
     if(n>0)
         w+=n;
     return(n);
