@@ -295,7 +295,7 @@ void Setup()
 int Run()
 {
 	fd_set rfds, wfds;
-	int maxfd;
+	int maxfd(0);
 	for(vector<int>::iterator it=vecSocks.begin(); it!=vecSocks.end(); it++)
 		if(*it > maxfd)
 			maxfd=*it;
