@@ -203,10 +203,12 @@ bool dlcon::_HandleIncoming(string & sErrorOut)
 	int r = m_InBuf.sysread(m_conFd);
 	ldbg("dlcon::_HandleIncoming()~sysread: " <<r);
 	if (r <= 0)
+	{
 		if(r==-EAGAIN /* retries done there || r==-EINTR */ )
 			return true;
 		else
 			return false;
+	}
 
 	Reswitch:
 	ldbg("switch: " << m_DlState);
@@ -414,7 +416,7 @@ void dlcon::MakeRequest(tDlJob* j) {
 	if (m_proxy) // add auth (in sPath) if possible and other stuff
 		head+=m_proxy->sPath+"Proxy-Connection: keep-alive\r\n";
 
-	if (j->m_pStorage->m_nFileSize > 0)
+	if (j->m_pStorage->m_nSizeSeen > 0)
 	{
 		bool bSetRange(false), bSetIfRange(false);
 		
@@ -422,7 +424,7 @@ void dlcon::MakeRequest(tDlJob* j) {
 		const header *pHead = j->m_pStorage->GetHeader();
 		const char *p=pHead->h[header::LAST_MODIFIED];
 		
-		if (j->m_pStorage->m_bIsDynType)
+		if (j->m_pStorage->m_bCheckFreshness)
 		{
 			if (p)
 			{
@@ -447,7 +449,7 @@ void dlcon::MakeRequest(tDlJob* j) {
 			 * send at least one byte if the assumed position is correct, and we never get a
 			 * 416 error (one byte waste is acceptable). */
 			char buf[50];
-			sprintf(buf, "Range: bytes=%ld-\r\n", j->m_pStorage->m_nFileSize - 1 );
+			sprintf(buf, "Range: bytes=%ld-\r\n", j->m_pStorage->m_nSizeSeen - 1 );
 			head+=buf;
 		}
 		if (bSetIfRange)
@@ -456,7 +458,6 @@ void dlcon::MakeRequest(tDlJob* j) {
 			head += p;
 			head += "\r\n";
 		}
-
 	}
 	// Debian Apt-Cacher/" ACVERSION 
 	head+="User-Agent: " + acfg::agentname + "\r\n\r\n";
