@@ -26,14 +26,14 @@
 using namespace MYSTD;
 
 expiration::expiration(int fd) :
-	maintenance(fd), now(0), nDropPrefLen(0), m_nErrorCount(0), m_repFileStream(NULL)
+	maintenance(fd), m_nTimeNow(0), m_nDropPrefLen(0), m_bErrAbort(false),
+	m_nErrorCount(0), m_repFileStream(NULL)
 {
 	m_szDecoFile="maint.html";
 }
 
 expiration::~expiration()
 {
-
 	if(m_repFileStream)
 	{
 		fclose(m_repFileStream);
@@ -52,7 +52,7 @@ void expiration::ProcessRegular(const string & sPath, const struct stat &)
 
 	rechecks::eFileKind kind;
 	if(sPath.length()<5 || sPath.substr(sPath.length()-5, 5)!=".head")
-		kind = rex.getFiletype(sPath.c_str());
+		kind = m_rex.getFiletype(sPath.c_str());
 	else
 		kind = rechecks::FILE_PKG; // threat .head like package files here, will need to cleanup as well
 
@@ -60,16 +60,19 @@ void expiration::ProcessRegular(const string & sPath, const struct stat &)
 	{
 		tStrPos pos=sPath.rfind(sPathSep[0]);
 		// must be there... 
-
-		// First file will shadow all others with the same base name!
 		
-		// don't double search even on misses, detect when STL initialized it
-		pair<time_t,string> & val = m_trash[sPath.substr(pos+1)];
-		if (val.first==0)
+		pair<string, pair<time_t,string> > entry(sPath.substr(pos+1), 
+						pair<time_t,string>(m_nTimeNow, sPath.substr(m_nDropPrefLen, pos-m_nDropPrefLen)) );
+		// deal with multimap, scan the range and look for previous entry for this file
+		pair<trashmap_t::iterator, trashmap_t::iterator> fromTo=
+			m_trashCandidates.equal_range(entry.first);
+		for(trashmap_t::iterator it=fromTo.first; it!=fromTo.second; it++)
 		{
-			val.first=now;
-			val.second=sPath.substr(nDropPrefLen, pos-nDropPrefLen);
+			if(it->second.second==entry.second.second)
+				return; // seen this already
 		}
+		// not similar found in the range
+		m_trashCandidates.insert(entry);
 	}
 	else if (rechecks::FILE_INDEX == kind 
 			&& sPath.compare(acfg::cachedir.size()+1, 7, "_import"))
@@ -98,7 +101,8 @@ void expiration::_LoadTrashMapFromFile()
 	string sLine;
 	
 	// stuff for user info
-	time_t oldest(0), newest(0);
+	time_t now=time(NULL);
+	time_t oldest(now), newest(0);
 	
 	while(reader.GetOneLine(sLine))
 	{
@@ -106,146 +110,184 @@ void expiration::_LoadTrashMapFromFile()
 		if(Tokenize(sLine, "\t", elems)==3)
 		{
 			time_t timestamp=atoll(elems[0].c_str());
-			m_trash[elems[2]]=pair<time_t,string>(timestamp, elems[1]);
+
+			m_trashCandidates.insert(pair<string, pair<time_t,string> > (elems[2], 
+					pair<time_t,string>(timestamp, elems[1]) ) );
 			
-			if(oldest==0)
-			{
-				oldest=newest=timestamp;
-			}
-			else
-			{
-				if(timestamp < oldest)
-					oldest=timestamp;
-				if(timestamp > newest)
-					newest=timestamp;
-			}
+			if(timestamp < oldest)
+				oldest=timestamp;
+			if(timestamp > newest)
+				newest=timestamp;
 		}
 	}
 	/*
 	cout << "Unreferenced: ";
-	for(trashmap_t::iterator it=m_trash.begin(); it!=m_trash.end(); it++)
+	for(trashmap_t::iterator it=m_trashCandidates.begin(); it!=m_trashCandidates.end(); it++)
 		fprintf(stdout, "%lu\t%s\t%s\n",  it->second.first, it->second.second.c_str(), it->first.c_str());
 	*/
 	
-	if(m_trash.size()>0)
+	if(m_trashCandidates.size()>0)
 	{
 		// pretty printing for impatient users
 		char buf[200];
 		
-		// fix range, map to days behind
-		time_t now=time(0);
-		if(oldest>now)
-			oldest=now;
-		if(newest>now)
-			newest=now;
-		oldest=(now-oldest)/86400;
-		newest=(now-newest)/86400;
-		oldest = (oldest>acfg::extreshhold ? oldest-acfg::extreshhold : 0);
-		newest = (newest>acfg::extreshhold ? newest-acfg::extreshhold : 0);
-		if(oldest==newest)
-			oldest++;
-		const char *szTheS=(oldest>1)?"s":"";
+		// map to to wait
+		int nMax=acfg::extreshhold-((now-oldest)/86400);
+		int nMin=acfg::extreshhold-((now-newest)/86400);
 				
 		snprintf(buf, _countof(buf), "Previously detected: %lu rotten package file(s), "
-				"to be deleted in about %lu-%lu day%s<br>\n", 
-				(unsigned long) m_trash.size(),
-				newest, oldest, szTheS);
+				"to be deleted in about %d-%d day(s)<br>\n", 
+				(unsigned long) m_trashCandidates.size(),
+				nMin, nMax==nMin?nMax+1:nMax); // cheat a bit for the sake of code simplicity
 		SendMsg(buf);
 	}
 }
 
-
-inline void expiration::_UpdateVolatileFiles()
+bool expiration::_DownloadOne(dlcon &dl, const MYSTD::string & sFilePath, string &sErr)
 {
+	tFileItemPtr fi;
+	acfg::tHostiVec *pBackends(NULL);
 
-	// TODO: implement pdiff reconstruction
-	
-	for(tStrVec::iterator it=m_volatileFiles.begin(); it!=m_volatileFiles.end(); it++)
-	{
-		
-		// keep related .head files safe for now
-		tStrPos pos=it->rfind(*szPathSep);
-		m_trash.erase( it->substr(pos+1)+".head" );
-		
-		
-		dlcon dl;
-		tFileItemPtr fi;
-		acfg::tHostiVec *pBackends(NULL);
-		
-		string sErr;
-		string sRepName, sFilePath;
-		
-		string sKeyPath=it->substr(acfg::cachedir.size()+1, stmiss);
+	string sKeyPath=sFilePath.substr(acfg::cachedir.size()+1, stmiss);
 
-		// keypath must represent a previosly resolved URL, being including either repname or real host which works
-		tHttpUrl url;
+	// keypath must represent a previosly resolved URL, being including either repname or real host which works
+	tHttpUrl url;
 #ifdef WIN32
 #error rewrite for the backslashes or change reliably in the walker to slashes
 #endif
-		if(!url.SetHttpUrl(sKeyPath))
-		{
-			sErr=sKeyPath+" does not contain a valid repository or host name.";
+	if (!url.SetHttpUrl(sKeyPath))
+	{
+		sErr=sKeyPath+" does not contain a valid repository or host name.";
+		goto rep_update_prob;
+	}
+
+	fi=GetFileItem(sKeyPath);
+	if (fi)
+		fi->Setup(true);
+	else
+	{
+		sErr=" could not create file item handler.";
+		goto rep_update_prob;
+	}
+
+	SendMsg(string("Checking/Updating ")+sKeyPath+"...<br>\n");
+	pBackends = acfg::GetBackendVec(&url.sHost);
+	if (pBackends)
+		url.sPath.erase(0, 1); // get only the path string residual
+	else
+	{ // try something different, do we have the original URL?
+		const header *pHead=fi->GetHeader();
+		// must be well formated to be valid
+		if (pHead && pHead->h[header::XORIG] && 0==strncmp(
+				pHead->h[header::XORIG], "http://", 7)
+				&& url.SetHttpUrl(pHead->h[header::XORIG]))
+			pBackends = acfg::GetBackendVec(&url.sHost);
+	}
+	if (pBackends)
+		dl.AddJob(fi, pBackends, url.sPath);
+	else
+		// use as hostname, and pray
+		dl.AddJob(fi, url);
+
+	dl.WorkLoop(true);
+	{
+		lockguard g(*fi);
+		if (fi->status == FIST_ERROR)
 			goto rep_update_prob;
-		}
-			
-		fi=GetFileItem(sKeyPath);
-		if(fi)
-			fi->Setup(true);
-		else
-		{
-			sErr=" could not create file item handler.";
-			goto rep_update_prob;
-		}
-		
-		SendMsg(string("Checking/Updating ")+sKeyPath+"...<br>\n");
-		pBackends = acfg::GetBackendVec(& url.sHost);
-		if (pBackends)
-			url.sPath.erase(0, 1); // get only the path string residual
-		else 
-		{ // try something different, do we have the original URL?
-			const header *pHead=fi->GetHeader();
-			// must be well formated to be valid
-			if (pHead && pHead->h[header::XORIG] && 
-					0==strncmp(pHead->h[header::XORIG], "http://", 7) &&
-					url.SetHttpUrl(pHead->h[header::XORIG]))
-				pBackends = acfg::GetBackendVec(&url.sHost);
-		}
-		if(pBackends)
-			dl.AddJob(fi, pBackends, url.sPath);
-		else // use as hostname, and pray
-			dl.AddJob(fi, url);
-		
-		dl.WorkLoop(true);
+	}
+
+	if (fi)
+	{
+		DelUser(sKeyPath);
+		fi.reset();
+	}
+
+	return true;
+
+	rep_update_prob:
+
+	if (fi)
+	{
 		{
 			lockguard g(*fi);
-			if(fi->status == FIST_ERROR)
-				goto rep_update_prob;
+			const header *pErHead=fi->GetHeader();
+			if (pErHead)
+				sErr+=string("Reason: ")+pErHead->frontLine+"<br>";
 		}
-		
-		if(fi)
+		DelUser(sKeyPath);
+		fi.reset();
+	}
+	return false;
+}
+
+inline void expiration::_UpdateVolatileFiles()
+{
+	// TODO: implement pdiff reconstruction
+	dlcon dl;
+	tStrVec extraFiles;
+
+	for(tStrVec::const_iterator it=m_volatileFiles.begin(); it!=m_volatileFiles.end(); it++)
+	{	
+		string sErr;
+		if (!_DownloadOne(dl, *it, sErr))
 		{
-			DelUser(sKeyPath);
-			fi.reset();
+			m_nErrorCount++;
+			SendMsg("Error while redownloading this file!<br> ");
+			if (!sErr.empty())
+				SendMsg(string("Reason: ")+sErr+"<br>");
 		}
 		
-		continue;
-		
-		rep_update_prob:
-		m_nErrorCount++;
-		
-		SendMsg("Error while redownloading this file!<br> ");
-		if (fi)
+		// last but not least: some voodoo, make sure we have the full version
+		string path=*it;
+		tStrPos pos=path.find("Packages.diff");
+		if(pos!=stmiss)
+			path.replace(pos+9, path.length()-pos+9, "bz2");
+		else if(pos=path.find("Sources.diff"), pos!=stmiss)
+			path.replace(pos+8, path.length()-pos+8, "bz2");	
+		else
+			continue;
+		// candidate of that already seen in some queue?
+		string gzVersion=path;
+		gzVersion.replace(path.length()-3, 3, "gz");
+		for(tStrVec::const_iterator it=extraFiles.begin(); it!=extraFiles.end(); it++)
 		{
-			{
-				lockguard g(*fi);
-				const header *pErHead=fi->GetHeader();
-				if(pErHead)
-					SendMsg(string("Reason: ")+pErHead->frontLine+"<br>");
-			}
-			DelUser(sKeyPath);
-			fi.reset();
+			if(path==*it || gzVersion==*it)
+				goto do_next_ifile;
 		}
+		for(tStrVec::const_iterator it=m_volatileFiles.begin(); it!=m_volatileFiles.end(); it++)
+		{	
+			if(path==*it || gzVersion==*it)
+				goto do_next_ifile;
+		}
+		extraFiles.push_back(path);
+		extraFiles.push_back(gzVersion);
+		do_next_ifile:
+		;
+	}
+	if(!extraFiles.empty())
+		SendMsg("Trying to download index files from guessed locations...<br>");
+	for(tStrVec::const_iterator it=extraFiles.begin(); it!=extraFiles.end(); it++)
+	{
+		if(it->empty())
+			continue;
+				
+		string sErr;
+		const string &path=*it;
 		
+		if(_DownloadOne(dl, path, sErr))
+		{
+			if(path[path.size()-1]=='2')
+				it++; // skip .gz version if .bz2 was downloaded
+			SendMsg(" ...OK!<br>\n");
+			m_volatileFiles.push_back(path);
+		}
+		else
+		{
+			SendMsg("Error while redownloading this file!<br> ");
+			if(!sErr.empty())
+				SendMsg(string("Reason: ")+sErr+"<br>");
+		}
+			
 	}
 }
 
@@ -308,7 +350,7 @@ inline void expiration::_ParseVolatileFilesAndHandleEntries()
 			SendMsg("<font color=red>An error occured while reading this file, some contents may have been ignored.</font><br>");
 			continue;
 		}
-//		cout << "found package files: "<< m_trash.size()<<endl;
+//		cout << "found package files: "<< m_trashCandidates.size()<<endl;
 	}
 }
 
@@ -318,23 +360,29 @@ void expiration::_HandlePkgEntry(const string &sReferer, const NoCaseStringMap &
 	NoCaseStringMap::const_iterator it=info.find("Filename");
 	if(it==info.end())
 		return;
-	const string & sFilename=it->second;
-	tStrPos pos=sFilename.rfind(*SZPATHSEP);
-	// might have path components, act only on basename
-	if(pos!=stmiss)
-	{
-		m_trash.erase(sFilename.substr(pos+1));
-		m_trash.erase(sFilename.substr(pos+1)+".head");
-	}
-	else
-	{
-		m_trash.erase(sFilename);
-		m_trash.erase(sFilename+".head");
-	}
+	
+	tStrPos pos=it->second.rfind(*SZPATHSEP);
+	string sFilename=(pos==stmiss?it->second: it->second.substr(pos+1));
+
+	trashmap_t::iterator itr;
+	pair<trashmap_t::iterator, trashmap_t::iterator> fromTo;
+	fromTo=m_trashCandidates.equal_range(sFilename);
+	for(trashmap_t::iterator itr=fromTo.first; itr!=fromTo.second; itr++)
+		m_trashCandidates.erase(itr);
+	fromTo=m_trashCandidates.equal_range(sFilename+".head");
+	for(trashmap_t::iterator itr=fromTo.first; itr!=fromTo.second; itr++)
+		m_trashCandidates.erase(itr);
 }
 
-inline void expiration::_RemoveAndStoreStatus()
+inline void expiration::_RemoveAndStoreStatus(bool bPurgeNow)
 {
+	{ // those files have nothing to referencing them
+		pair<trashmap_t::iterator, trashmap_t::iterator> fromTo;
+		fromTo=m_trashCandidates.equal_range("Release.gpg.head");
+		for (trashmap_t::iterator itr=fromTo.first; itr!=fromTo.second; itr++)
+			m_trashCandidates.erase(itr);
+	}
+		
 	string sDbFile=acfg::cachedir+sPathSep+"_expending_dat";
 	FILE *f = fopen(sDbFile.c_str(), "w");
 	if(!f)
@@ -354,11 +402,12 @@ inline void expiration::_RemoveAndStoreStatus()
 	time_t now=time(NULL);
 	
 	//cout << "Unreferenced: ";
-	for(trashmap_t::iterator it=m_trash.begin(); it!=m_trash.end(); it++)
+	int n(0);
+	for(trashmap_t::iterator it=m_trashCandidates.begin(); it!=m_trashCandidates.end(); it++)
 	{
 		//if(acfg::debug)
 		//	cerr << "Vgl: " << it->second.first << " < " << (now-acfg::extreshhold*86400) <<endl; 
-		if(it->second.first < (now-acfg::extreshhold*86400))
+		if(bPurgeNow || (it->second.first < (now-acfg::extreshhold*86400)))
 		{
 			string sWhat=acfg::cachedir+sPathSep+it->second.second+sPathSep+it->first;
 			SendMsg(string("Removing ")+it->second.second+sPathSep+it->first+"<br>\n");
@@ -366,23 +415,28 @@ inline void expiration::_RemoveAndStoreStatus()
 			unlink((sWhat+".head").c_str());
 		}
 		else
+		{
+			n++;
 			fprintf(f, "%lu\t%s\t%s\n",  it->second.first, it->second.second.c_str(), it->first.c_str());
+		}
 	}
 	fclose(f);
-	
+	char buf[100];
+	snprintf(buf, _countof(buf), "<br>%d package file(s) marked for removal in few days.<br><br>\n", n);
+	SendMsg(buf);
 }
 
 
 
-void expiration::Action(const string &)
+void expiration::Action(const string & cmd)
 {
 	_LoadTrashMapFromFile();
 	SendMsg("Locating potentially expired files in the cache...<br>\n");
 
-	nDropPrefLen=acfg::cachedir.size()+1;
-	now=time(NULL);
+	m_nDropPrefLen=acfg::cachedir.size()+1;
+	m_nTimeNow=time(NULL);
 	DirectoryWalk(acfg::cachedir, this);
-	cout << "found package files: " << m_trash.size()<<endl;
+	cout << "found package files: " << m_trashCandidates.size()<<endl;
 
 	SendMsg("Redownloading index files...<br>\n");
 	_UpdateVolatileFiles();
@@ -394,7 +448,7 @@ void expiration::Action(const string &)
 	}
 	SendMsg("Identifying fresh files...<br>\n");
 	_ParseVolatileFilesAndHandleEntries();
-	_RemoveAndStoreStatus();
+	_RemoveAndStoreStatus(cmd.find("purgeNow")!=stmiss);
 	_PurgeMaintLogs();
 	SendMsg("Done.");
 
@@ -429,6 +483,9 @@ void expiration::DumpLog(time_t id)
 
 void expiration::Run(const string &cmd)
 {
+
+	m_bErrAbort=(cmd.find("abortOnErrors=aOe")!=stmiss);
+
 	SendDecoration(true, "200", m_szDecoFile);
 
 	static lockable onlyOneCanRun;
