@@ -1,9 +1,12 @@
 
+// make sure the right function strerror_r is used
+#define _XOPEN_SOURCE 600
+#include <string.h>
+
 #define LOCAL_DEBUG
 #include "debug.h"
 
 #include "config.h"
-
 #include "fileitem.h"
 #include "header.h"
 #include "acfg.h"
@@ -23,10 +26,11 @@ header const * fileitem::GetHeader()
 
 fileitem::fileitem(MYSTD::string sPath) :
 	condition(),
-	m_bIsDynType(true),
+	m_bCheckFreshness(true),
 	status(FIST_FRESH),
 	m_nIncommingCount(0),
-	m_nFileSize(0),	
+	m_nSizeChecked(0),
+	m_nSizeSeen(0),	
 	m_sPath(acfg::cachedir+sPathSep+sPath),
 	m_sKey(sPath),
 	m_filefd(-1)
@@ -52,14 +56,14 @@ int fileitem::GetFileFd() {
 #ifdef HAS_ADVISE
 	// optional, experimental
 	if(status==FIST_COMPLETE)
-		posix_fadvise(fd, 0, m_nFileSize, POSIX_FADV_SEQUENTIAL);
+		posix_fadvise(fd, 0, m_nSizeChecked, POSIX_FADV_SEQUENTIAL);
 #endif
 
 	return fd;
 }
 
 
-FiStatus fileitem::Setup(bool bIsDynType) {
+FiStatus fileitem::Setup(bool bCheckFreshness) {
 	
 	setLockGuard;
 	if(status>FIST_FRESH)
@@ -67,7 +71,7 @@ FiStatus fileitem::Setup(bool bIsDynType) {
 	
 	status=FIST_INITED;
 	
-	m_bIsDynType = bIsDynType;
+	m_bCheckFreshness = bCheckFreshness;
 	
 	if(m_head.LoadFromFile(m_sPath+".head") >0 && m_head.type==header::ANSWER )
 	{
@@ -76,35 +80,66 @@ FiStatus fileitem::Setup(bool bIsDynType) {
 		
 		struct stat stbuf;
 		if (0==::stat(m_sPath.c_str(), &stbuf))
-			m_nFileSize=stbuf.st_size;
-					
-		// some optimizations and plausibility checks
-		const char *p=m_head.h[header::CONTENT_LENGTH];
-		// should always have it, even in chunked mode it stored during fixation
-		if(!p) goto error_clean;
+			m_nSizeSeen=stbuf.st_size;
 		
-		off_t nContLen=atol(p);
-		// accept length for non-volatile files, are they complete?
-		if(!bIsDynType && stbuf.st_size == nContLen) 
-			status=FIST_COMPLETE;
-		else if(nContLen< m_nFileSize)
-			goto error_clean;
+		// some plausibility checks
+		if(m_bCheckFreshness)
+		{
+			const char *p=m_head.h[header::LAST_MODIFIED];
+			if(!p)
+				goto error_clean; // suspicious, cannot use it
+		}
+		else
+		{ // non-volatile files, so could accept the length, do some checks first
 			
-		// can never be set finished (unchecked) unless in offline mode
-		if(bIsDynType && !acfg::offlinemode)
-			status=FIST_INITED;
-		
+			const char *pContLen=m_head.h[header::CONTENT_LENGTH];
+			if(pContLen)
+			{
+				off_t nContLen=atol(pContLen); // if it's 0 then we assume it's 0
+				
+				// file larger than it could ever be?
+				if(nContLen < m_nSizeSeen)
+					goto error_clean;
+				
+				// accept length for non-volatile files...
+				m_nSizeChecked=m_nSizeSeen;
+				// ... are they even complete? a 0 value also looks weird, try to continue
+				if(m_nSizeSeen == nContLen && nContLen>0)
+					status=FIST_COMPLETE;	
+			}
+			else
+			{
+				// no content length known, assume it's ok
+				m_nSizeChecked=m_nSizeSeen;
+			}				
+		}
 	}
 	return status;
 
 	error_clean:
 			unlink((m_sPath+".head").c_str());
 			m_head.clear();
-			m_nFileSize=0;
+			m_nSizeSeen=0;
 			status=FIST_INITED;
 			return status; // unuseable, to be redownloaded
 }
 
+void _LogWithErrno(const char *msg, const string & sFile)
+{
+	char buf[32];
+	buf[0]=buf[31]=0x0;
+	const char *szErr=&buf[0];
+
+#ifdef _GNU_SOURCE
+#warning GLIBC BUG DETECTED -- _GNU_SOURCE is set in C++ mode
+	szErr=strerror_r(errno, buf, sizeof(buf)-1);
+#else
+	strerror_r(errno, buf, sizeof(buf)-1);
+#endif
+	
+	aclog::err( (sFile+" storage error ["+msg+"]: "+szErr).c_str());
+}
+
 bool fileitem::StoreHeader(const header & h)
 {
 	setLockGuard;
@@ -118,22 +153,25 @@ bool fileitem::StoreHeader(const header & h)
 	
 	// status will change, most likely... ie. BOUNCE action
 	notifyAll();
-	
-#define BOUNCE(x) { m_head.frontLine="HTTP/1.1 "; m_head.frontLine+=x; status=FIST_ERROR; return false; }
+
+#define SETERROR(x) { m_head.frontLine="HTTP/1.1 "; m_head.frontLine+=x; status=FIST_ERROR; _LogWithErrno(x, m_sPath); }
+#define SETERRORKILLFILE(x) { SETERROR(x); goto kill_file; }
+#define BOUNCE(x) { SETERROR(x); _LogWithErrno(x, m_sPath); return false; }
 	
 	int code=h.getStatus();
 	ldbg("http code: "<< code);
 	bool bStoreData(false);
+	string sHeadPath=m_sPath+".head";
 	
-	if(code == 200)
+	if(200 == code)
 	{
 		bStoreData=true;
-		m_nFileSize=0;
+		m_nSizeChecked=0;
 		m_head=h;
 	}
-	else if(code==206)
+	else if(206 == code)
 	{
-		if(m_nFileSize<=0)
+		if(m_nSizeSeen<=0)
 		{
 			// wtf? Cannot have requested partial content
 			BOUNCE("500 Unexpected Partial Response");
@@ -155,11 +193,11 @@ bool fileitem::StoreHeader(const header & h)
 		ldbg("resuming? n: "<< n << " und myfrom: " <<myfrom << 
 				" und myto: " << myto << " und mylen: " << mylen);
 		if(n!=3  // check for nonsense
-				|| myfrom != m_nFileSize-1
+				|| myfrom != m_nSizeSeen-1
 				|| myfrom<0 || mylen<0)
 			BOUNCE("500 Server reports illegal range");
 	
-		m_nFileSize--;
+		m_nSizeChecked=myfrom;
 		bStoreData=true;
 		
 		m_head=h;
@@ -167,65 +205,111 @@ bool fileitem::StoreHeader(const header & h)
 		m_head.del(header::CONTENT_RANGE);
 		m_head.set(header::CONTENT_LENGTH, mylen);
 	}
-	else if(h.getStatus()!=304 && m_head.getStatus()==200 && m_head.type==header::ANSWER)
+	else if(416 == code) // that's always bad; it cannot be complete (-1 trick) -> kill cached file ASAP
 	{
-		// just do nothing
-		bStoreData=false;
+		SETERRORKILLFILE("503 Server disagrees on file size, cleaning up");
 	}
 	else
 	{
 		bStoreData=false;
-		// have a clear one with just the message
+		// have a clear one with just the error message
 		m_head.frontLine=h.frontLine;
 		m_head.type=header::ANSWER;
+		
 	}
-	
+			
 	if(bStoreData)
 	{
-		string path=m_sPath+".head";
-		mkbasedir(path);
-		ldbg("Opening "+path);
-		int count=m_head.StoreToFile(path);
+		// using adaptive Delete-Or-Replace-Or-CopyOnWrite strategy
+		
+		// First opening the file first to be sure that it can be written. Header storage is the critical point,
+		// every error after that leads to full cleanup to not risk inconsistent file contents 
 		
-		// unlink and retry
-		if(count<0)
-			unlink(path.c_str());
-		count=m_head.StoreToFile(path);
-
-		if(count<0)
-		{
-			if(-count!=ENOSPC)
-			{
-				aclog::err(string("Error writting header: ")+m_sPath+".head");
-				BOUNCE("503 Cache storage error");
-			}
-			else 
-				BOUNCE("503 OUT OF DISK SPACE");
-		}
-
 		int flags = O_WRONLY | O_CREAT;
-		
+		struct stat stbuf;
+				
+		mkbasedir(m_sPath);
 		m_filefd=open(m_sPath.c_str(), flags, 00644);
 		ldbg("file opened?! returned: " << m_filefd);
 		
 		// self-recovery from cache poisoned with files with wrong permissions
 		if (m_filefd<0)
 		{
-			unlink(m_sPath.c_str());
-			m_filefd=open(m_sPath.c_str(), flags, 00644);
-			ldbg("file force-opened?! returned: " << m_filefd);
+			if(m_nSizeChecked>0) // OOOH CRAP! CANNOT APPEND HERE! Do what's still possible.
+			{
+				string temp=m_sPath+".tmp";
+				if(FileCopy(m_sPath, temp) && 0==unlink(m_sPath.c_str()) )
+				{
+					if(0!=rename(temp.c_str(), m_sPath.c_str()))
+						BOUNCE("503 Cannot rename files");
+					
+					// be sure about that
+					if(0!=stat(m_sPath.c_str(), &stbuf) || stbuf.st_size!=m_nSizeSeen)
+						BOUNCE("503 Cannot copy file parts, filesystem full?");
+					
+					m_filefd=open(m_sPath.c_str(), flags, 00644);
+					ldbg("file opened after copying around: ");
+				}
+				else
+					BOUNCE("503 Cannot store or remove files");
+			}
+			else
+			{
+				unlink(m_sPath.c_str());
+				m_filefd=open(m_sPath.c_str(), flags, 00644);
+				ldbg("file force-opened?! returned: " << m_filefd);
+			}
 		}
 		
 		if (m_filefd<0)
 			BOUNCE(errno==ENOSPC ? "503 OUT OF DISK SPACE" : 
-			"503 Cache storage error, opening data file.");
+			"503 Cache storage error while opening data file");
 		
-		if(0!=ftruncate(m_filefd, m_nFileSize) || m_nFileSize!=lseek(m_filefd, m_nFileSize, SEEK_SET))
-			BOUNCE("503 IO error, positioning");
+		if(0!=fstat(m_filefd, &stbuf) || !S_ISREG(stbuf.st_mode))
+			SETERRORKILLFILE("503 Not a regular file");
+		
+		// crop, but only if the new size is smaller. MUST NEVER become larger (would fill with zeros)
+		if(m_nSizeChecked < m_nSizeSeen)
+		{
+			if(0!=ftruncate(m_filefd, m_nSizeChecked))
+				SETERRORKILLFILE("503 Cannot change file size");
+		}
+		else if(m_nSizeChecked>m_nSizeSeen) // should never happen and caught by the checks above
+			SETERRORKILLFILE("503 Internal error on size checking");
+		
+		ldbg("Opening "+sHeadPath);
+		int count=m_head.StoreToFile(sHeadPath);
+		
+		// unlink and retry
+		if(count<0)
+			unlink(sHeadPath.c_str());
+		count=m_head.StoreToFile(sHeadPath);
+
+		if(count<0)
+			SETERRORKILLFILE( (-count!=ENOSPC) ? "503 Cache storage error" : "503 OUT OF DISK SPACE");
+			
+		// double-check the sane state
+		if(0!=fstat(m_filefd, &stbuf) || stbuf.st_size!=m_nSizeChecked)
+			SETERRORKILLFILE("503 Inconsistent file state");
+			
+		if(m_nSizeChecked!=lseek(m_filefd, m_nSizeChecked, SEEK_SET))
+			SETERRORKILLFILE("503 IO error, positioning");
 	}
 	
 	status=FIST_DLGOTHEAD;
 	return true;
+
+	kill_file:
+	if(m_filefd>=0)
+	{
+		close(m_filefd);
+		m_filefd=-1;
+	}
+	unlink(m_sPath.c_str());
+	unlink(sHeadPath.c_str());
+	
+	status=FIST_ERROR;
+	return false;
 }
 
 bool fileitem::StoreFileData(const char *data, unsigned int size) {
@@ -247,7 +331,7 @@ bool fileitem::StoreFileData(const char *data, unsigned int size) {
 		// we are done! Fix header from chunked transfers?
 		if (m_filefd>=0 && ! m_head.h[header::CONTENT_LENGTH])
 		{
-			m_head.set(header::CONTENT_LENGTH, m_nFileSize);
+			m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
 			m_head.StoreToFile(m_sPath+".head");
 		}
 	}
@@ -268,7 +352,7 @@ bool fileitem::StoreFileData(const char *data, unsigned int size) {
 						BOUNCE("503 OUT OF DISK SPACE");
 					BOUNCE("503 General storage error");
 				}
-				m_nFileSize+=r;
+				m_nSizeChecked+=r;
 				size-=r;
 				data+=r;
 			}
