@@ -247,7 +247,6 @@ void job::PrepareDownload() {
     	goto report_invpath;
     }
     
-        
     bForceFreshnessChecks = ( ! acfg::offlinemode && m_type==rechecks::FILE_INDEX);
     m_pItem=GetFileItem(m_sFileLoc);
     
@@ -263,10 +262,9 @@ void job::PrepareDownload() {
     
     if(acfg::offlinemode) { // make sure there will be no problems later in SendData or prepare a user message
     	if(fistate==FIST_COMPLETE)
-    		return; // perfect
-    	if(fistate>=FIST_ERROR)
-    		goto report_offlineconf;
-    	return; // either way, we are done here
+    		return; // perfect, done here
+    	// error and/or needs download, but freshness check was disabled, so it's really not complete.
+    	goto report_offlineconf;
     }
     dbgline;
     if( fistate < FIST_DLGOTHEAD) // needs a downloader 
@@ -363,7 +361,7 @@ eJobResult job::SendData(int confd)
 		while (true)
 		{
 			fistate=m_pItem->status;
-			nNewSize=m_pItem->m_nFileSize;
+			nNewSize=m_pItem->m_nSizeChecked;
 			
 			if (fistate>=FIST_ERROR)
 			{
@@ -454,6 +452,8 @@ eJobResult job::SendData(int confd)
 							bool bGotLen(false);
 
 							bool bFreshnessForced=(m_type!=rechecks::FILE_INDEX || m_pReqHead->h[header::XORIG]);
+							// this plays well with lighttpd <-> Apt, but may not work with the alternative time format
+							// TODO: add format tolerant time comparer
 							bool bFreshnessTested=(pIfmo && m_RespHead.h[header::LAST_MODIFIED] &&
 									0==strcmp(pIfmo, m_RespHead.h[header::LAST_MODIFIED]));
 							
@@ -520,8 +520,9 @@ eJobResult job::SendData(int confd)
 								+"\r\n";
 							}
 						}
-						
-											
+						if(m_RespHead.h[header::LAST_MODIFIED])
+							m_sPrependBuf+=string("Last-Modified: ")
+							+m_RespHead.h[header::LAST_MODIFIED]+"\r\n";
 					}
 					m_sPrependBuf+=header::GetInfoHeaders();
 					if(bSendBody)
@@ -569,7 +570,7 @@ eJobResult job::SendData(int confd)
 					ldbg("STATE_SEND_PLAIN_DATA untill " << nNewSize);
 
 					// eof?
-					if(m_nSendPos==m_pItem->m_nFileSize && fistate>=FIST_COMPLETE)
+					if(m_nSendPos==m_pItem->m_nSizeChecked && fistate>=FIST_COMPLETE)
 						return m_bCloseCon ? R_DISCON : R_DONE;
 					
 					if(nNewSize>m_nRangeLast)
@@ -705,8 +706,15 @@ eJobResult job::SendData(int confd)
 			{
 				m_sPrependBuf=string("HTTP/1.1 ")+m_sErrorMsgBuffer
 				+ "\r\nConnection: " + (m_bCloseCon?"close\r\n":"Keep-Alive\r\n")
-				+ header::GetInfoHeaders() +
-				+ "X-Original-Source: " + m_sFileLoc + "\r\n\r\n";
+				+ header::GetInfoHeaders()+"X-Original-Source: " /*;
+				
+				if(m_RespHead.h[header::XORIG])
+					m_sPrependBuf+=m_RespHead.h[header::XORIG];
+				else
+					m_sPrependBuf+=m_sFileLoc;
+				m_sPrependBuf+= "\r\n\r\n";*/
+				+ (m_RespHead.h[header::XORIG]?m_RespHead.h[header::XORIG]:m_sFileLoc)
+				+ "\r\n\r\n";
 				m_backstate=STATE_ERRORCONT;
 				m_sErrorMsgBuffer.clear();
 				USRDBG(REPLEVEL, "Response error or message, will continue (for " << m_sFileLoc<<")");
