@@ -7,6 +7,7 @@
 #include "expiration.h"
 #include "pkgimport.h"
 #include "showinfo.h"
+#include "reportgen.h"
 #include "aclogger.h"
 #include "filereader.h"
 #include "acfg.h"
@@ -30,7 +31,6 @@ using namespace MYSTD;
 #endif
 
 maintenance::maintenance(int fd) :
-	m_bErrAbort(false),
 	m_reportFD(fd),
 	m_szDecoFile(NULL)
 {
@@ -175,8 +175,6 @@ void DispatchAndRunMaintTask(const MYSTD::string &cmd, int conFD, const char * s
 		else
 			pWorker = new showinfo(conFD);
 
-		pWorker->m_bErrAbort=(cmd.find("abortOnErrors=aOe")!=stmiss);
-	
 		pWorker->Run(cmd);
 	}
 	MYCATCH(...)
