@@ -24,7 +24,7 @@ int getUUID() {
 */
 void set_nb(int fd) {
     int flags = fcntl(fd, F_GETFL);
-    ASSERT(flags != -1);
+    //ASSERT(flags != -1);
     flags |= O_NONBLOCK;
     flags = fcntl(fd, F_SETFL, flags);
 }
