@@ -35,7 +35,7 @@ void pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 {
 		//string sRelName=fileNode->sPath.substr(acfg::cachedir.length()+1);
 		
-		rechecks::eFileKind kind = rex.getFiletype(sPath.c_str());
+		rechecks::eFileKind kind = m_rex.getFiletype(sPath.c_str());
 
 		if( rechecks::FILE_INDEX == kind && m_bPickIfiles
 				&& sPath.compare(acfg::cachedir.size()+1, 7, "_import"))
@@ -45,118 +45,112 @@ void pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 		}
 		else if (rechecks::FILE_PKG == kind && !m_bPickIfiles)
 		{
-#ifdef SUCKS_MAY_DO_USELESS_WORK_AND_FAILS_AT_READONLY_FS_CREATE_HEADER_WHEN_NEEDED
-			//cout << "Is new pkgfile\n";
-			string sHeadPath=sPath+".head";
-			//tStrPos nSlashPos=sPath.rfind(*SZPATHSEP);
-			header h;
-			if (h.LoadFromFile(sHeadPath) >0)
-			{
-				//cerr <<"Found valid .head file"<<endl;
-				;
-			}
-			else
-			{
-#if INCOMPLETE_AC_SUPPORT
-				string sTestPath(sPath);
-				sTestPath.replace(nSlashPos, 1,
-						SZPATHSEP".."SZPATHSEP"headers"SZPATHSEP);
 
-				//cerr << "checking "<< sTestPath<<endl;
-				if (h.LoadFromFile(sTestPath)>0)
-				{
-					// APTCACHER SOURCE!
-					
-					string sFlagPath(sPath);
-					sFlagPath.replace(nSlashPos, 1,
-							SZPATHSEP".."SZPATHSEP"private"SZPATHSEP);
-					sFlagPath+=".complete";
-					acbuf reader;
-					if(reader.initFromFile(sFlagPath.c_str()))
-					{
-						string s(reader.rptr(), reader.size());
-						job::FixUri(s);
-						if(!s.empty())
-							//cerr<<"Got original URI: " << s<<endl;
-					}
-					/*
-						m_pParent->SaveMove(sTestPath, sTargetDir+sPath.substr(nSlashPos)+".head");
-						m_pParent->SaveMove(sPath, sTargetDir+sPath.substr(nSlashPos));
-						return;
-					*/
-					m_nVerCount++;
-				}
-				else
-#endif
-				
-					h.fix();
-					h.frontLine="HTTP/1.1 200 OK";
-					h.set("Content-Length", stinfo.st_size);
-					// don't! h.set("X-Original-Source", string("imported, ")
-					//		+sPath.substr(nDropPrefLen));
-					h.type=header::ANSWER;
-					if (h.StoreToFile(sHeadPath)<=0)
-					{
-						aclog::err("Unable to store generated header");
-						return; // TODO: wirklich?
-					}
-			}
-#endif
 			// get a fingerprint, either by checksumming or from fpr cache
-			tFingerprint fpr;
-			tCmapIter cacheIt;
-			
-			//cout << "any cached key for: " << sPath << ", "<< stinfo.st_size 
-			//	<< ";" << stinfo.st_mtime << "?"<<endl;
-			
-			if( bUseCache && m_cacheMap.end() != (cacheIt
-					=m_cacheMap.find(tFInfo(sPath, stinfo.st_size, stinfo.st_mtime))) )
+		tFingerprint fpr;
+		tCmapIter cacheIt;
+
+		//cout << "any cached key for: " << sPath << ", "<< stinfo.st_size 
+		//	<< ";" << stinfo.st_mtime << "?"<<endl;
+
+		if (bUseCache && m_cacheMap.end() != (cacheIt =m_cacheMap.find(tFInfo(
+				sPath, stinfo.st_size, stinfo.st_mtime))))
+		{
+			fpr=cacheIt->second;
+			SendMsg(string("<font color=blue>Checked ")+sPath
+					+" (fingerprint from cache)</font><br>\n");
+		}
+		else
+		{
+			if (filereader::GetMd5Sum(sPath, fpr.sum))
 			{
-				fpr=cacheIt->second;
+				fpr.size=stinfo.st_size;
 				SendMsg(string("<font color=blue>Checked ")+sPath
-						+" (fingerprint from cache)</font><br>\n");
+						+" (fingerprint created)</font><br>\n");
 			}
 			else
 			{
-				if(filereader::GetMd5Sum(sPath, fpr.sum))
-				{
-					fpr.size=stinfo.st_size;
-					SendMsg(string("<font color=blue>Checked ")+sPath
-							+" (fingerprint created)</font><br>\n");
-				}
-				else
-				{
-					SendMsg(string("<font color=red>Error checking ")+sPath
-							+"</font><br>\n");
-					return;
-				}
-				
-			}
-			
-			// add this entry immediately if needed and get its reference
-			tFInfo & node = m_importFileMap[fpr];
-			 
-			if(node.sPath.empty())
-			{ // fresh, just added to the map
-				node.sPath=sPath;
-				node.fsize=stinfo.st_size;
-				node.mtime=stinfo.st_mtime;
-			}
-			else // is already there, must be a duplicate
-			{
-				// add the new one as deletion candidate with a different/incorrect key
-				fpr.size*=2;
-				tFInfo & dupe = m_importFileMap[fpr];
-				dupe.bFileUsed = true;
+				SendMsg(string("<font color=red>Error checking ")+sPath
+						+"</font><br>\n");
+				return;
 			}
+
+		}
+
+		// add this entry immediately if needed and get its reference
+		tFInfo & node = m_importFileMap[fpr];
+
+		if (node.sPath.empty())
+		{ // fresh, just added to the map
+			node.sPath=sPath;
+			node.fsize=stinfo.st_size;
+			node.mtime=stinfo.st_mtime;
+		}
+		else // is already there, must be a duplicate
+		{
+			// add the new one as deletion candidate with a different/incorrect key
+			fpr.size*=2;
+			tFInfo & dupe = m_importFileMap[fpr];
+			dupe.bFileUsed = true;
 		}
 	}
+}
 
+bool FileCopy(const string &from, const string to)
+{
+	// not possible? different filesystems?
+	acbuf buf;
+	buf.init(50000);
+	int in(-1), out(-1);
 
+	in=open(from.c_str(), O_RDONLY);
+	if (in<0) // error, here?!
+		return false;
+
+	while (true)
+	{
+		int err;
+		err=buf.sysread(in);
+		if (err<0)
+		{
+			if (err==-EAGAIN || err==-EINTR)
+				continue;
+			else
+				goto error_copying;
+		}
+		else if (err==0)
+			break;
+		// don't open unless the input is readable, for sure
+		if (out<0)
+		{
+			out=open(to.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 00644);
+			if (out<0)
+				goto error_copying;
+		}
+		err=buf.syswrite(out);
+		if (err<=0)
+		{
+			if (err==-EAGAIN || err==-EINTR)
+				continue;
+			else
+				goto error_copying;
+		}
+
+	}
+
+	close(in);
+	close(out);
+	return true;
+
+	error_copying: if (in>=0)
+		close(in);
+	if (out>=0)
+		close(out);
+	return false;
+}
 bool pkgimport::LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
 {
 	mkbasedir(to);
-	int in(-1), out(-1);
 	
 	//ldbg("Moving " <<from<< " to " << to);
 	
@@ -191,55 +185,16 @@ bool pkgimport::LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
 			return true;
 
 		SendMsg("(couldn't link, copying contents)");
-		// not possible? different filesystems?
-		acbuf buf;
-		buf.init(50000);
-		
-		in=open(from.c_str(), O_RDONLY);
-		if(in<0) // error, here?!
+		if(!FileCopy(from,to))
 			return false;
 		
-		while (true)
-		{
-			int err;
-			err=buf.sysread(in);
-			if (err<0)
-				if (err==-EAGAIN)
-					continue;
-				else
-					goto error_copying;
-			else if (err==0)
-        break;
-			// don't open unless the input is readable, for sure
-			if(out<0)
-			{
-				out=open(to.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 00644);
-				if(out<0)
-					goto error_copying;
-			}
-			err=buf.syswrite(out);
-			if (err<=0)
-				if (err==-EAGAIN)
-					continue;
-				else
-					goto error_copying;
-			
-		}
-		close(in);
-		close(out);
+		// make sure the copy is ok
+		return (0 == stat(from.c_str(), &stinfo) 
+				&& 0 == stat(to.c_str(), &toinfo)
+				&& toinfo.st_size==stinfo.st_size);
 		
-		// if the copy is ok, remove the source and we are done
-		off_t size;
-		if ( 0 == stat(from.c_str(), &stinfo)&& (size=stinfo.st_size, true)&& 0
-				== stat(to.c_str(), &stinfo)&& size==stinfo.st_size) {
-			return true;
-		}
 	}
-	error_copying:
-	if(in>=0)
-		close(in);
-	if(out>=0)
-		close(out);
+
 	return false;
 }
 
@@ -282,14 +237,14 @@ void pkgimport::Action(const string &cmd)
 	
 	SendMsg(string("Importing from ")+m_sSrcPath+" directory, scanning...<br>\n");
 	
-	nDropPrefLen=1+m_sSrcPath.length();
+	m_nDropPrefLen=1+m_sSrcPath.length();
 	
 	_LoadKeyCache();
 	
 	m_bPickIfiles=true;
 	DirectoryWalk(acfg::cachedir, this);
 	UINT piCount(0), siCount(0);
-	for(tStrVec::iterator it=m_volatileFiles.begin();it!=m_volatileFiles.end();it++)
+	for(tStrVec::const_iterator it=m_volatileFiles.begin();it!=m_volatileFiles.end();it++)
 	{
 		piCount+=(stmiss!=it->find("Packages"));
 		siCount+=(stmiss!=it->find("Sources"));
