@@ -11,9 +11,9 @@ rechecks::rechecks() {
     int rc;
     rc=regcomp(&rePfiles, acfg::pfilepat.c_str(), REG_EXTENDED);
     ASSERT(!rc);
-    rc=regcomp(&reIfiles, acfg::ifilepat.c_str(), REG_EXTENDED);
+    rc=regcomp(&reIfiles, acfg::vfilepat.c_str(), REG_EXTENDED);
     ASSERT(!rc);
-    ldbg("reIfiles: " << rc);
+    //ldbg("reIfiles: " << rc);
     //rc=regcomp(&rePoison, "[^a-zA-Z0-9\\-+_~./]", REG_EXTENDED);
     //rc=regcomp(&rePoison, "[^a-zA-Z0-9._~+/-]", REG_EXTENDED);
     //ldbg("rePoison: " << rc);
@@ -37,15 +37,6 @@ rechecks::eFileKind rechecks::getFiletype(const char *in) {
     return FILE_INVALID;
 }
 
-/*
-bool rechecks::hasPoison(const char *in) {
-    //ldbg("Poisoned? " << in);
-    if(!in)
-        return true;
-    return(!regexec(&rePoison, in, 0, NULL, 0));
-}
-*/
-
 /*
 int main(int argc, char **argv) {
     rechecks checka;
