@@ -1,5 +1,5 @@
-#include "../include/showinfo.h"
 
+#include "showinfo.h"
 #include "meta.h"
 #include "acbuf.h"
 #include "acfg.h"
@@ -48,7 +48,7 @@ void showinfo::Run(const string &)
 	{
 		string contents(pagebuf.rptr(), pagebuf.size());
 		SetStyle(contents);
-		StrSubst(contents, "$SERVERIP", GetHostname());
+		StrSubst(contents, "$SERVERIP", string("[")+GetHostname()+"]");
 		StrSubst(contents, "$SERVERPORT", acfg::port.c_str());
 		StrSubst(contents, "$REPAGE", sPathSepUnix+acfg::reportpage);
 		
